/*
 * Tuxcall -- Call waiting detector for Linux
 * Copyright (C) 2001-2002 Angel Luis Jimenez Martinez
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#   include <config.h>
#endif

#include <intl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <linux/soundcard.h>
#include "vec.h"
#include "iir.h"
#include "dsp.h"
#include "detect.h"
#include "conf.h"
#include "common.h"
#include "aux.h"

#define CONF "tuxcallrc"


/* Globals */
int Log= 0 /* Log activated? */
  , ChunkSize= 640 /* Sample group's size */
  , CheckPPP= 1; /* Does it need to check for PPP connection? */
float Freq= 420; /* Call waiting tone frequency */
char HangCmd[CMDLEN]= "killall -9 pppd" /* Command for hanging connection */
  ,Device[CMDLEN]= "/dev/dsp"; /* Sound input device */
  

/*
 * Function executed when a call is detected
 */
void onCall()
{
    mySystem( HangCmd );
}

void use()
{
    printf( _("TuxCall - version %s\n\n"), VERSION );
    printf( _("use: tuxcall [-l] [-n] [-f frequency] [-e command] [-d device]\n") );
    printf( _("   -l              Store filtered signals and mean on files\n                   f2-detect.dat and f2-mean.dat\n") );
    printf( _("   -n              Don't check for ppp connection.\n") );
    printf( _("   -f frequency    Call waiting tone frequency (Hertz). (Default value %.0f)\n"), Freq );
    printf( _("   -e \"command\"    Command executed for hanging connection\n                   when a call is detected.\n") );
    printf( _("   -d device       Device for reading sound input. (Default /dev/dsp)\n") );
}

/*
 * Parse program arguments
 */
void parseArgs( int argc, char ** argv )
{
    int i;

    for ( i=1; i<argc; i++ )
    {
        if ( strcmp(argv[i], "-l") == 0 )
        {
            Log= 1;
        }
        else if ( strcmp(argv[i], "-n") == 0 )
        {
            CheckPPP= 0;
        }
        else if ( strcmp(argv[i], "-f") == 0 )
        {
            if ( i< argc-1 )
            {
                Freq= atof( argv[++i] );
            }
            else
            {
                use();
                exit(-1);
            }
        }    
        else if ( strcmp(argv[i], "-e") == 0 )
        {
            if ( i< argc-1 )
            {
                strncpy( HangCmd, argv[++i], CMDLEN-1 );
                HangCmd[CMDLEN-1]= 0;
            }
            else
            {
                use();
                exit(-1);
            }
        }    
        else if ( strcmp(argv[i], "-d") == 0 )
        {
            if ( i< argc-1 )
            {
                strncpy( Device, argv[++i], CMDLEN-1 );
                HangCmd[CMDLEN-1]= 0;
            }
            else
            {
                use();
                exit(-1);
            }
        }    
        else    
        {
            use();
            exit(-1);
        }
    }    
}

int main( int argc, char ** argv )
{
    float a[3], b[3];
    float center, bw;
    signed short * buff;
    float * filtered;
    char confFile[FNAMELEN];
    FILE * f;
    int dsp;

    bw= 0.00125;

#ifdef ENABLE_NLS
    setlocale(LC_ALL, "");
    bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
    textdomain(PACKAGE);
#endif    

    /* Loading global config */
    snprintf( confFile, FNAMELEN-1, "/etc/%s", CONF );
    f= fopen( confFile, "r" );
    if ( f != NULL )
    {
    	loadConfig( f, Device, HangCmd, &Freq );
        fclose( f );
    }

    /* Loading user config */
    snprintf( confFile, FNAMELEN-1, "%s/.%s", getenv("HOME"), CONF );
    f= fopen( confFile, "r" );
    if ( f != NULL )
    {
    	loadConfig( f, Device, HangCmd, &Freq );
        fclose( f );
    }

    parseArgs( argc, argv );

    center= Freq/8000;

    allocateBuff( &buff, &filtered, BUFFLEN );

    iirPassband( a, b, center, bw );

#ifdef DEBUG    
    printf( _("device= %s\n"), Device );
    printf( _("command= %s\n"), HangCmd );
    printf( _("frequency= %f\n"), Freq );
    printf( "center= %f, bw= %f\n", center, bw );
    printf( "a0= %40.43f\na1= %40.43f\na2= %40.43f\nb1= %40.43f\nb2= %40.43f\n"
        , a[0], a[1], a[2], b[1], b[2] );
#endif         

    fprintf( stderr, _("Waiting for ppp connection... ") );
    fflush( stderr );

    while ( 1 )
    {
        if ( connected() || !CheckPPP )
        {
            fprintf( stderr, _("detected.\n") );

            printf( _("Opening %s... "), Device );
            fflush( stdout );
            dsp= initDSP( Device, FORMAT, CHANNELS, SPEED );
            if ( dsp == -1 )
            {
                fprintf( stderr, _("Error opening %s...\n"), Device );
                exit( -1 );
            }
            printf( _("done.\n") );
            fprintf( stderr, _("Detecting...") );
            fprintf( stderr, "\n" );

            while ( detectStep( dsp, buff, filtered, BUFFLEN, a, b, NULL
                    , onCall ) );

            close( dsp );

            fprintf( stderr, _("Waiting for ppp connection... ") );
            fflush( stderr );
        }    
        else
            sleep( 5 );
    }    


    freeBuff( buff, filtered );

    return 0;
}







